@@ -489,7 +489,9 @@ MOS_STATUS CodechalDebugInterface::DumpYUVSurface(
     PMOS_SURFACE              surface,
     const char *              attrName,
     const char *              surfName,
-    CODECHAL_MEDIA_STATE_TYPE mediaState)
+    CODECHAL_MEDIA_STATE_TYPE mediaState,
+    uint32_t                  width_in,
+    uint32_t                  height_in)
 {
     if (!DumpIsEnabled(attrName, mediaState))
     {
@@ -499,13 +501,22 @@ MOS_STATUS CodechalDebugInterface::DumpYUVSurface(
     MOS_LOCK_PARAMS lockFlags;
     MOS_ZeroMemory(&lockFlags, sizeof(MOS_LOCK_PARAMS));
     lockFlags.ReadOnly = 1;
+    lockFlags.TiledAsTiled = 1; // Bypass GMM CPU blit due to some issues in GMM CpuBlt function
 
-    uint8_t *surfBaseAddr = (uint8_t *)m_osInterface->pfnLockResource(m_osInterface, &surface->OsResource, &lockFlags);
+    uint8_t *lockedAddr = (uint8_t *)m_osInterface->pfnLockResource(m_osInterface, &surface->OsResource, &lockFlags);
 
-    surfBaseAddr += surface->dwOffset + surface->YPlaneOffset.iYOffset * surface->dwPitch;
-    uint8_t *data   = surfBaseAddr;
-    uint32_t width  = surface->dwWidth;
-    uint32_t height = surface->dwHeight;
+    // Always use MOS swizzle instead of GMM Cpu blit
+    uint32_t sizeMain = (uint32_t)(surface->OsResource.pGmmResInfo->GetSizeMainSurface());
+    uint8_t *surfBaseAddr = (uint8_t*)MOS_AllocMemory(sizeMain);
+
+    CODECHAL_DEBUG_CHK_NULL(surfBaseAddr);
+    Mos_SwizzleData(lockedAddr, surfBaseAddr, surface->TileType, MOS_TILE_LINEAR, sizeMain / surface->dwPitch, surface->dwPitch, 0);
+
+    uint8_t *data = surfBaseAddr;
+    data += surface->dwOffset + surface->YPlaneOffset.iYOffset * surface->dwPitch;
+
+    uint32_t width  = width_in ? width_in : surface->dwWidth;
+    uint32_t height = height_in? height_in : surface->dwHeight;
 
     switch (surface->Format)
     {
@@ -518,6 +529,7 @@ MOS_STATUS CodechalDebugInterface::DumpYUVSurface(
     case Format_Y216:
     case Format_Y210:  //422 10bit -- Y0[15:0]:U[15:0]:Y1[15:0]:V[15:0] = 32bits per pixel = 4Bytes per pixel
     case Format_Y410:  //444 10bit -- A[31:30]:V[29:20]:Y[19:10]:U[9:0] = 32bits per pixel = 4Bytes per pixel
+    case Format_R10G10B10A2:
     case Format_AYUV:  //444 8bit  -- A[31:24]:Y[23:16]:U[15:8]:V[7:0] = 32bits per pixel = 4Bytes per pixel
         width = width << 2;
         break;
@@ -566,9 +578,9 @@ MOS_STATUS CodechalDebugInterface::DumpYUVSurface(
         height >>= 1;
         break;
     case  Format_Y416:
-    case  Format_AYUV:
     case  Format_AUYV:
     case  Format_Y410: //444 10bit
+    case  Format_R10G10B10A2:
         height *= 2;
         break;
     case  Format_YUY2:
@@ -586,6 +598,7 @@ MOS_STATUS CodechalDebugInterface::DumpYUVSurface(
     case Format_IMC3:
         height = height / 2;
         break;
+    case  Format_AYUV:
     default:
         height = 0;
         break;
@@ -632,6 +645,7 @@ uint8_t *vPlaneData = surfBaseAddr;
     if (surfBaseAddr)
     {
         m_osInterface->pfnUnlockResource(m_osInterface, &surface->OsResource);
+        MOS_FreeMemory(surfBaseAddr);
     }
 
     return MOS_STATUS_SUCCESS;
@@ -749,16 +763,17 @@ MOS_STATUS CodechalDebugInterface::DumpSurface(
     lockFlags.ReadOnly = 1;
     uint8_t *data      = (uint8_t *)m_osInterface->pfnLockResource(m_osInterface, &surface->OsResource, &lockFlags);
     CODECHAL_DEBUG_CHK_NULL(data);
-
+    
+    std::string bufName  = std::string(surfaceName) + "_w[" + std::to_string(surface->dwWidth) + "]_h[" + std::to_string(surface->dwHeight) + "]_p[" + std::to_string(surface->dwPitch) + "]";
     const char *fileName;
     if (mediaState == CODECHAL_NUM_MEDIA_STATES)
     {
-        fileName = CreateFileName(surfaceName, nullptr, extType);
+        fileName = CreateFileName(bufName.c_str(), nullptr, extType);
     }
     else
     {
         std::string kernelName = m_configMgr->GetMediaStateStr(mediaState);
-        fileName               = CreateFileName(kernelName.c_str(), surfaceName, extType);
+        fileName               = CreateFileName(kernelName.c_str(), bufName.c_str(), extType);
     }
 
     MOS_STATUS status;
@@ -867,6 +882,9 @@ MOS_STATUS CodechalDebugInterface::DumpHucDmem(
     case hucRegionDumpHpu:
         funcName = funcName + dmemName + "_HpuPass" + passName;
         break;
+    case hucRegionDumpBackAnnotation:
+        funcName = funcName + dmemName + "_BackAnnotationPass" + passName;
+        break;
     default:
         funcName = funcName + dmemName + "_Pass" + passName;
         break;
@@ -937,6 +955,9 @@ MOS_STATUS CodechalDebugInterface::DumpHucRegion(
     case hucRegionDumpHpu:
         funcName = funcName + inputName + bufName + regionNumName + regionName + "_HpuPass" + passName;
         break;
+    case hucRegionDumpBackAnnotation:
+        funcName = funcName + inputName + bufName + regionNumName + regionName + "_BackAnnotationPass" + passName;
+        break;
     default:
         funcName = funcName + inputName + bufName + regionNumName + regionName + "_Pass" + passName;
         break;
@@ -945,6 +966,13 @@ MOS_STATUS CodechalDebugInterface::DumpHucRegion(
     return DumpBuffer(region, nullptr, funcName.c_str(), regionSize, regionOffset);
 }
 
+MOS_STATUS CodechalDebugInterface::DumpBltOutput(
+    PMOS_SURFACE              surface,
+    const char *              attrName)
+{
+    return MOS_STATUS_SUCCESS;
+}
+
 MOS_STATUS CodechalDebugInterface::DeleteCfgLinkNode(uint32_t frameIdx)
 {
    return m_configMgr->DeleteCfgNode(frameIdx);
