@@ -121,7 +121,7 @@ MOS_STATUS CodechalDecode::AllocateSurface(
     uint32_t        height,
     const char      *name,
     MOS_FORMAT      format,
-    bool            isCompressed)
+    bool            isCompressible)
 {
     MOS_STATUS eStatus = MOS_STATUS_SUCCESS;
 
@@ -139,7 +139,7 @@ MOS_STATUS CodechalDecode::AllocateSurface(
     allocParams.dwHeight    = height;
     allocParams.dwArraySize = 1;
     allocParams.pBufName    = name;
-    allocParams.bIsCompressed = isCompressed;
+    allocParams.bIsCompressible = isCompressible;
 
     CODECHAL_DECODE_CHK_STATUS_MESSAGE_RETURN(m_osInterface->pfnAllocateResource(
         m_osInterface,
@@ -229,6 +229,8 @@ CodechalDecode::CodechalDecode (
 {
     CODECHAL_DECODE_FUNCTION_ENTER;
 
+    MOS_ZeroMemory(&m_dummyReference, sizeof(MOS_SURFACE));
+
     CODECHAL_DECODE_CHK_NULL_NO_STATUS_RETURN(hwInterface);
     CODECHAL_DECODE_CHK_NULL_NO_STATUS_RETURN(hwInterface->GetOsInterface());
     CODECHAL_DECODE_CHK_NULL_NO_STATUS_RETURN(hwInterface->GetMiInterface());
@@ -251,8 +253,6 @@ CodechalDecode::CodechalDecode (
 
     m_mode              = standardInfo->Mode;
     m_isHybridDecoder   = standardInfo->bIsHybridCodec ? true : false;
-
-    MOS_ZeroMemory(&m_dummyReference, sizeof(MOS_SURFACE));
 }
 
 MOS_STATUS CodechalDecode::SetGpuCtxCreatOption(
@@ -868,15 +868,6 @@ MOS_STATUS CodechalDecode::VerifySpaceAvailable ()
     return eStatus;
 }
 
-MOS_STATUS CodechalDecode::BeginFrame ()
-{
-    CODECHAL_DECODE_FUNCTION_ENTER;
-
-    m_firstExecuteCall = true;
-
-    return MOS_STATUS_SUCCESS;
-}
-
 MOS_STATUS CodechalDecode::EndFrame ()
 {
     MOS_STATUS eStatus = MOS_STATUS_SUCCESS;
@@ -1100,12 +1091,16 @@ MOS_STATUS CodechalDecode::EndFrame ()
 MOS_STATUS CodechalDecode::Execute(void *params)
 {
     MOS_STATUS eStatus = MOS_STATUS_SUCCESS;
+    
+    PERF_UTILITY_AUTO(__FUNCTION__, PERF_DECODE, PERF_LEVEL_HAL);
 
     CODECHAL_DECODE_FUNCTION_ENTER;
 
     CODECHAL_DECODE_CHK_STATUS_RETURN(Codechal::Execute(params));
 
     CodechalDecodeParams *decodeParams = (CodechalDecodeParams *)params;
+    m_executeCallIndex = decodeParams->m_executeCallIndex;
+
     // MSDK event handling
     Mos_Solo_SetGpuAppTaskEvent(m_osInterface, decodeParams->m_gpuAppTaskEvent);
 
@@ -1122,12 +1117,9 @@ MOS_STATUS CodechalDecode::Execute(void *params)
     {
         CODECHAL_DECODE_CHK_STATUS_RETURN(Mos_Solo_DisableAubcaptureOptimizations(
             m_osInterface,
-            m_firstExecuteCall));
+            IsFirstExecuteCall()));
     }
 
-    // For multiple execution call, this function will be entered by multple times,
-    // so clear bFirstExecuteCall flag when this function exit.
-    CodechalDecodeRestoreData<bool> FirstExecuteCallRestore(&(m_firstExecuteCall), false);
 #ifdef _DECODE_PROCESSING_SUPPORTED
     if (decodeParams->m_refFrameCnt != 0)
     {
@@ -1242,7 +1234,7 @@ MOS_STATUS CodechalDecode::Execute(void *params)
         if (decodeParams->m_dataBuffer &&
             (m_standard != CODECHAL_JPEG && m_cencBuf == nullptr) &&
             !(m_standard == CODECHAL_HEVC && m_isHybridDecoder) &&
-            !(m_standard == CODECHAL_HEVC && (m_incompletePicture || !m_firstExecuteCall)))
+            !(m_standard == CODECHAL_HEVC && (m_incompletePicture || !IsFirstExecuteCall())))
         {
             if (m_mode == CODECHAL_DECODE_MODE_MPEG2VLD ||
                 m_mode == CODECHAL_DECODE_MODE_VC1VLD   ||
@@ -1749,9 +1741,10 @@ MOS_STATUS CodechalDecode::SendPrologWithFrameTracking(
         cmdBuffer->Attributes.dwMediaFrameTrackingAddrOffset = 0;
     }
 
-#ifdef _MMC_SUPPORTED
-    CODECHAL_DECODE_CHK_STATUS_RETURN(m_mmc->SendPrologCmd(m_miInterface, cmdBuffer, MOS_RCS_ENGINE_USED(gpuContext)));
-#endif
+    if (m_mmc)
+    {
+        CODECHAL_DECODE_CHK_STATUS_RETURN(m_mmc->SendPrologCmd(m_miInterface, cmdBuffer, gpuContext));
+    }
 
     MHW_GENERIC_PROLOG_PARAMS genericPrologParams;
     MOS_ZeroMemory(&genericPrologParams, sizeof(genericPrologParams));
