@@ -1548,7 +1548,8 @@ struct CodechalEncodeAvcEnc : public CodechalEncodeAvcBase
     //!
     virtual MOS_STATUS SendPrologWithFrameTracking(
         PMOS_COMMAND_BUFFER         cmdBuffer,
-        bool                        frameTracking) override;
+        bool                        frameTracking,
+        MHW_MI_MMIOREGISTERS       *mmioRegister = nullptr) override;
 
     //!
     //! \brief  Realize the scene change report
@@ -1653,14 +1654,14 @@ struct CodechalEncodeAvcEnc : public CodechalEncodeAvcBase
 
 #if USE_CODECHAL_DEBUG_TOOL
 protected:
-    virtual MOS_STATUS DumpSeqParFile();
-    virtual MOS_STATUS DumpFrameParFile();
+    virtual MOS_STATUS DumpSeqParFile()  override;
+    virtual MOS_STATUS DumpFrameParFile() override;
 
     virtual MOS_STATUS PopulateHmeParam(
         bool    is16xMeEnabled,
         bool    is32xMeEnabled,
         uint8_t meMethod,
-        void    *cmd);
+        void    *cmd) override;
 #endif
 };
 #endif  // __CODECHAL_ENCODE_AVC_H__
