@@ -25,6 +25,7 @@
 //!
 #include "codechal_hw.h"
 #include "codechal_setting.h"
+#include "mhw_cmd_reader.h"
 
 #define VDBOX_HUC_VDENC_BRC_INIT_KERNEL_DESCRIPTOR 4
 
@@ -345,6 +346,126 @@ MOS_STATUS CodechalHwInterface::GetHxxPrimitiveCommandSize(
     return eStatus;
 }
 
+MOS_STATUS CodechalHwInterface::GetHcpStateCommandSize(
+    uint32_t                        mode,
+    uint32_t *                      commandsSize,
+    uint32_t *                      patchListSize,
+    PMHW_VDBOX_STATE_CMDSIZE_PARAMS params)
+{
+    MOS_STATUS eStatus = MOS_STATUS_SUCCESS;
+
+    CODECHAL_HW_FUNCTION_ENTER;
+
+    uint32_t standard = CodecHal_GetStandardFromMode(mode);
+
+    uint32_t hcpCommandsSize  = 0;
+    uint32_t hcpPatchListSize = 0;
+    uint32_t cpCmdsize        = 0;
+    uint32_t cpPatchListSize  = 0;
+
+    if (m_hcpInterface && (standard == CODECHAL_HEVC || standard == CODECHAL_VP9))
+    {
+        CODECHAL_HW_CHK_STATUS_RETURN(m_hcpInterface->GetHcpStateCommandSize(
+            mode, &hcpCommandsSize, &hcpPatchListSize, params));
+
+        m_cpInterface->GetCpStateLevelCmdSize(cpCmdsize, cpPatchListSize);
+    }
+
+    *commandsSize  = hcpCommandsSize + cpCmdsize;
+    *patchListSize = hcpPatchListSize + cpPatchListSize;
+
+    return eStatus;
+}
+
+MOS_STATUS CodechalHwInterface::GetHcpPrimitiveCommandSize(
+    uint32_t                        mode,
+    uint32_t                       *commandsSize,
+    uint32_t                       *patchListSize,
+    bool                            modeSpecific)
+{
+    MOS_STATUS eStatus = MOS_STATUS_SUCCESS;
+
+    CODECHAL_HW_FUNCTION_ENTER;
+
+    uint32_t standard = CodecHal_GetStandardFromMode(mode);
+
+    uint32_t hcpCommandsSize  = 0;
+    uint32_t hcpPatchListSize = 0;
+    uint32_t cpCmdsize        = 0;
+    uint32_t cpPatchListSize  = 0;
+
+    if (m_hcpInterface && (standard == CODECHAL_HEVC || standard == CODECHAL_VP9))
+    {
+        CODECHAL_HW_CHK_STATUS_RETURN(m_hcpInterface->GetHcpPrimitiveCommandSize(
+            mode, &hcpCommandsSize, &hcpPatchListSize, modeSpecific ? true : false));
+
+        m_cpInterface->GetCpSliceLevelCmdSize(cpCmdsize, cpPatchListSize);
+    }
+
+    *commandsSize  = hcpCommandsSize + cpCmdsize;
+    *patchListSize = hcpPatchListSize + cpPatchListSize;
+
+    return eStatus;
+}
+
+MOS_STATUS CodechalHwInterface::GetHucStateCommandSize(
+    uint32_t                        mode,
+    uint32_t *                      commandsSize,
+    uint32_t *                      patchListSize,
+    PMHW_VDBOX_STATE_CMDSIZE_PARAMS params)
+{
+    MOS_STATUS eStatus = MOS_STATUS_SUCCESS;
+
+    CODECHAL_HW_FUNCTION_ENTER;
+
+    uint32_t standard         = CodecHal_GetStandardFromMode(mode);
+    uint32_t hucCommandsSize  = 0;
+    uint32_t hucPatchListSize = 0;
+    uint32_t cpCmdsize        = 0;
+    uint32_t cpPatchListSize  = 0;
+
+    if (m_hucInterface && (standard == CODECHAL_HEVC || standard == CODECHAL_CENC || standard == CODECHAL_VP9 || standard == CODECHAL_AVC))
+    {
+        CODECHAL_HW_CHK_STATUS_RETURN(m_hucInterface->GetHucStateCommandSize(
+            mode, &hucCommandsSize, &hucPatchListSize, params));
+
+        m_cpInterface->GetCpStateLevelCmdSize(cpCmdsize, cpPatchListSize);
+    }
+
+    *commandsSize  = hucCommandsSize + cpCmdsize;
+    *patchListSize = hucPatchListSize + cpPatchListSize;
+
+    return eStatus;
+}
+
+MOS_STATUS CodechalHwInterface::GetHucPrimitiveCommandSize(
+    uint32_t                        mode,
+    uint32_t                       *commandsSize,
+    uint32_t                       *patchListSize)
+{
+    MOS_STATUS eStatus = MOS_STATUS_SUCCESS;
+
+    CODECHAL_HW_FUNCTION_ENTER;
+
+    uint32_t standard         = CodecHal_GetStandardFromMode(mode);
+    uint32_t hucCommandsSize  = 0;
+    uint32_t hucPatchListSize = 0;
+    uint32_t cpCmdsize        = 0;
+    uint32_t cpPatchListSize  = 0;
+
+    if (m_hucInterface && (standard == CODECHAL_HEVC || standard == CODECHAL_CENC || standard == CODECHAL_VP9))
+    {
+        CODECHAL_HW_CHK_STATUS_RETURN(m_hucInterface->GetHucPrimitiveCommandSize(
+            mode, &hucCommandsSize, &hucPatchListSize));
+        m_cpInterface->GetCpSliceLevelCmdSize(cpCmdsize, cpPatchListSize);
+    }
+
+    *commandsSize  = hucCommandsSize + cpCmdsize;
+    *patchListSize = hucPatchListSize + cpPatchListSize;
+
+    return eStatus;
+}
+
 MOS_STATUS CodechalHwInterface::GetVdencStateCommandsDataSize(
     uint32_t                    mode,
     uint32_t                   *commandsSize,
@@ -416,8 +537,8 @@ MOS_STATUS CodechalHwInterface::GetVdencPictureSecondLevelCommandsSize(
     {
         commands += m_hcpInterface->GetHcpVp9PicStateCommandSize();
         commands += m_hcpInterface->GetHcpVp9SegmentStateCommandSize() * 8;
-        commands += 124;
-        commands += 204;
+        commands += 128;
+        commands += 220;
         commands += m_sizeOfCmdBatchBufferEnd;
     }
     else
@@ -480,6 +601,8 @@ MOS_STATUS CodechalHwInterface::Initialize(
         }
     }
 
+    m_enableCodecMmc = !MEDIA_IS_WA(GetWaTable(), WaDisableCodecMmc) && settings->enableCodecMmc;
+
     return eStatus;
 }
 
@@ -512,6 +635,7 @@ MOS_STATUS CodechalHwInterface::AddVdencBrcImgBuffer(
     MOS_LOCK_PARAMS             lockFlags;
 
     uint32_t mfxAvcImgStateSize = m_mfxInterface->GetAvcImgStateSize();
+    uint32_t vdencAvcCostStateSize = m_vdencInterface->GetVdencAvcCostStateSize();
     uint32_t vdencAvcImgStateSize = m_vdencInterface->GetVdencAvcImgStateSize();
 
     MOS_ZeroMemory(&lockFlags, sizeof(MOS_LOCK_PARAMS));
@@ -524,21 +648,29 @@ MOS_STATUS CodechalHwInterface::AddVdencBrcImgBuffer(
 
     MOS_ZeroMemory(&constructedCmdBuf, sizeof(MOS_COMMAND_BUFFER));
     constructedCmdBuf.pCmdBase = (uint32_t *)data;
+    constructedCmdBuf.iRemaining = mfxAvcImgStateSize + vdencAvcCostStateSize + vdencAvcImgStateSize;
+
+    // Set MFX_IMAGE_STATE command
     constructedCmdBuf.pCmdPtr = (uint32_t *)data;
     constructedCmdBuf.iOffset = 0;
-    constructedCmdBuf.iRemaining = mfxAvcImgStateSize + vdencAvcImgStateSize;
-
     MHW_MI_CHK_STATUS(m_mfxInterface->AddMfxAvcImgCmd(&constructedCmdBuf, nullptr, params));
 
-    // Set VDENC_IMAGE_STATE command
+    // Set VDENC_COST_STATE command
     constructedCmdBuf.pCmdPtr = (uint32_t *)(data + mfxAvcImgStateSize);
     constructedCmdBuf.iOffset = mfxAvcImgStateSize;
+    m_vdencInterface->AddVdencAvcCostStateCmd(&constructedCmdBuf, nullptr, params);
+
+    // Set VDENC_IMAGE_STATE command
+    constructedCmdBuf.pCmdPtr = (uint32_t *)(data + mfxAvcImgStateSize + vdencAvcCostStateSize);
+    constructedCmdBuf.iOffset = mfxAvcImgStateSize + vdencAvcCostStateSize;
 
     MHW_MI_CHK_STATUS(m_vdencInterface->AddVdencImgStateCmd(&constructedCmdBuf, nullptr, params));
 
     // Add batch buffer end insertion flag
     m_miInterface->AddBatchBufferEndInsertionFlag(constructedCmdBuf);
 
+    OVERRIDE_CMD_DATA(constructedCmdBuf.pCmdBase, constructedCmdBuf.iOffset);
+
     if (data)
     {
         m_osInterface->pfnUnlockResource(
