@@ -125,6 +125,12 @@ MOS_USER_FEATURE_VALUE_ID pcComponentUserFeatureKeys[MOS_COMPONENT_COUNT][3] = {
     __MOS_USER_FEATURE_KEY_MESSAGE_MMC_TAG_ID,
     __MOS_USER_FEATURE_KEY_BY_SUB_COMPONENT_MMC_ID,
     __MOS_USER_FEATURE_KEY_SUB_COMPONENT_MMC_TAG_ID
+    },
+
+    {
+    __MOS_USER_FEATURE_KEY_MESSAGE_BLT_TAG_ID,
+    __MOS_USER_FEATURE_KEY_BY_SUB_COMPONENT_BLT_ID,
+    __MOS_USER_FEATURE_KEY_SUB_COMPONENT_BLT_TAG_ID
     }
 };
 
