@@ -59,6 +59,7 @@ void VphalFeatureReport::InitReportValue()
     CompositionMode     =   VPHAL_NO_COMPOSITION;
     DiScdMode           =   false;
     VEFeatureInUse      =   false;
+    HDRMode             =   VPHAL_HDR_MODE_NONE;
 }
 
 //!
@@ -105,7 +106,6 @@ MOS_STATUS VphalState::Allocate(
 
     if (MEDIA_IS_SKU(m_skuTable, FtrVERing) && m_veboxInterface)
     {
-        GpuNodeLimit.bCpEnabled        = (m_osInterface->osCpInterface->IsCpEnabled())? true : false;
         GpuNodeLimit.bSfcInUse         = MEDIA_IS_SKU(m_skuTable, FtrSFCPipe);
 
         // Check GPU Node decide logic together in this function
@@ -528,6 +528,10 @@ static MOS_STATUS VpHal_RenderWithAvsForMultiStreams(
             eStatus = eStatusSingleRender;
             VPHAL_PUBLIC_ASSERTMESSAGE("Failed to redner for substreams, eStatus: %d.\n", eStatus);
         }
+
+        //Free the temporary surface
+        pRenderer->GetOsInterface()->pfnFreeResource(pRenderer->GetOsInterface(), &(pIntermediateSurface->OsResource));
+
     }
 
 finish:
@@ -550,7 +554,6 @@ MOS_STATUS VphalState::Render(
     VPHAL_RENDER_PARAMS RenderParams;
 
     VPHAL_PUBLIC_CHK_NULL(pcRenderParams);
-
     RenderParams    = *pcRenderParams;
 
     if (VpHal_IsAvsSampleForMultiStreamsEnabled(pcRenderParams))
@@ -597,18 +600,18 @@ VphalState::VphalState(
         PMOS_INTERFACE          pOsInterface,
         PMOS_CONTEXT            pOsDriverContext,
         MOS_STATUS              *peStatus) :
+        m_gpuAppTaskEvent(nullptr),
         m_platform(),
         m_skuTable(nullptr),
         m_waTable(nullptr),
         m_osInterface(pOsInterface),
         m_renderHal(nullptr),
+        m_veboxInterface(nullptr),
         m_cpInterface(nullptr),
         m_sfcInterface(nullptr),
         m_renderer(nullptr),
-        m_veboxInterface(nullptr),
         m_renderGpuNode(MOS_GPU_NODE_3D),
-        m_renderGpuContext(MOS_GPU_CONTEXT_RENDER),
-        m_gpuAppTaskEvent(nullptr)
+        m_renderGpuContext(MOS_GPU_CONTEXT_RENDER)
 {
     MOS_STATUS                  eStatus;
 
@@ -681,10 +684,13 @@ VphalState::~VphalState()
 
     if (m_renderHal)
     {
-        eStatus = m_renderHal->pfnDestroy(m_renderHal);
-        if (eStatus != MOS_STATUS_SUCCESS)
+        if (m_renderHal->pfnDestroy)
         {
-            VPHAL_PUBLIC_ASSERTMESSAGE("Failed to destroy RenderHal, eStatus:%d.\n", eStatus);
+            eStatus = m_renderHal->pfnDestroy(m_renderHal);
+            if (eStatus != MOS_STATUS_SUCCESS)
+            {
+                VPHAL_PUBLIC_ASSERTMESSAGE("Failed to destroy RenderHal, eStatus:%d.\n", eStatus);
+            }
         }
         MOS_FreeMemory(m_renderHal);
     }
@@ -748,7 +754,7 @@ MOS_STATUS VphalState::GetStatusReport(
 {
     MOS_STATUS                     eStatus = MOS_STATUS_SUCCESS;
 
-#if (!EMUL && !ANDROID)        // this function is dummy for android and emul
+#if (!EMUL)        // this function is dummy for emul
     uint32_t                       i;
     uint32_t                       uiTableLen;
     PVPHAL_STATUS_TABLE            pStatusTable;
@@ -766,7 +772,7 @@ MOS_STATUS VphalState::GetStatusReport(
     // eNullRender = m_pOsInterface->pfnGetNullHWRenderFlags(m_pOsInterface);
 
     pOsContext           = m_osInterface->pOsContext;
-    pStatusTable         = &m_renderer->StatusTable;
+    pStatusTable         = &m_statusTable;
     uiNewHead            = pStatusTable->uiHead; // uiNewHead start from previous head value
     // entry length from head to tail
     uiTableLen           = (pStatusTable->uiCurrent - pStatusTable->uiHead) & (VPHAL_STATUS_TABLE_MAX_SIZE - 1);
@@ -781,6 +787,13 @@ MOS_STATUS VphalState::GetStatusReport(
         uiIndex            = (pStatusTable->uiHead + i) & (VPHAL_STATUS_TABLE_MAX_SIZE - 1);
         pStatusEntry       = &pStatusTable->aTableEntries[uiIndex];
 
+        // for tasks using CM, different streamIndexes may be used
+        uint32_t oldStreamIndex = m_osInterface->streamIndex;
+        if (pStatusEntry->isStreamIndexSet)
+        {
+            m_osInterface->streamIndex = pStatusEntry->streamIndex;
+        }
+
         if (bMarkNotReadyForRemains)
         {
             // the status is set as VPREP_NOTREADY while submitting commands
@@ -789,7 +802,7 @@ MOS_STATUS VphalState::GetStatusReport(
             continue;
         }
 
-#if LINUX
+#if (LINUX || ANDROID)
         dwGpuTag           = pOsContext->GetGPUTag(m_osInterface, pStatusEntry->GpuContextOrdinal);
 #else
         dwGpuTag           = pOsContext->GetGPUTag(pOsContext->GetGpuContextHandle(pStatusEntry->GpuContextOrdinal, m_osInterface->streamIndex));
@@ -816,11 +829,12 @@ MOS_STATUS VphalState::GetStatusReport(
         }
         else
         {   // here we have the first not ready entry.
-#if LINUX
+#if (LINUX || ANDROID)
             uiNewHead = (uiIndex + 1) & (VPHAL_STATUS_TABLE_MAX_SIZE - 1);
 #else
             uiNewHead = uiIndex;
 #endif
+
             bMarkNotReadyForRemains = true;
         }
 
@@ -831,6 +845,11 @@ MOS_STATUS VphalState::GetStatusReport(
 
         pQueryReport[i].dwStatus         = pStatusEntry->dwStatus;
         pQueryReport[i].StatusFeedBackID = pStatusEntry->StatusFeedBackID;
+
+        if (pStatusEntry->isStreamIndexSet)
+        {
+            m_osInterface->streamIndex = oldStreamIndex;
+        }
     }
     pStatusTable->uiHead = uiNewHead;
 
@@ -861,13 +880,12 @@ MOS_STATUS VphalState::GetStatusReportEntryLength(
     uint32_t*                      puiLength)
 {
     MOS_STATUS                     eStatus = MOS_STATUS_SUCCESS;
-#if __linux__  // this function is only for Linux now
+#if(!EMUL)        // this function is dummy for emul
     PVPHAL_STATUS_TABLE            pStatusTable;
 
-    VPHAL_PUBLIC_CHK_NULL(m_renderer);
     VPHAL_PUBLIC_CHK_NULL(puiLength);
 
-    pStatusTable = &m_renderer->StatusTable;
+    pStatusTable = &m_statusTable;
 
     // entry length from head to tail
     *puiLength = (pStatusTable->uiCurrent - pStatusTable->uiHead) & (VPHAL_STATUS_TABLE_MAX_SIZE - 1);
