@@ -165,6 +165,19 @@ bool VphalSfcState::IsOutputCapable(
     PVPHAL_SURFACE  renderTarget)
 {
     bool isOutputCapable = false;
+
+    VPHAL_RENDER_NORMALMESSAGE(
+        "isColorFill %d, \
+         src->rcDst.top %d, \
+         src->rcDst.left %d, \
+         renderTarget->TileType %d, \
+         renderTarget->Format %d",
+        isColorFill,
+        src->rcDst.top,
+        src->rcDst.left,
+        renderTarget->TileType,
+        renderTarget->Format);
+
     // H/W does not support ColorFill, the (OffsetX, OffsetY)
     // of scaled region not being (0, 0) or the tile type not being
     // Tile_Y on NV12/P010/P016 output surface. Disable SFC even if other
@@ -233,6 +246,28 @@ bool VphalSfcState::IsOutputPipeSfcFeasible(
     PVPHAL_SURFACE              pSrcSurface,
     PVPHAL_SURFACE              pRenderTarget)
 {
+    VPHAL_RENDER_NORMALMESSAGE(
+        "IsDisabled %d, \
+         uDstCount %d, \
+         Rotation %d, \
+         pTarget[0]->TileType %d, \
+         IsFormatSupported %d, \
+         InputFormat %d, \
+         OutputFormat %d, \
+         pCompAlpha %p, \
+         pDeinterlaceParams %p, \
+         bQueryVariance %d",
+        IsDisabled(),
+        pcRenderParams->uDstCount,
+        pSrcSurface->Rotation,
+        pcRenderParams->pTarget[0]->TileType,
+        IsFormatSupported(pSrcSurface, pcRenderParams->pTarget[0], pcRenderParams->pCompAlpha),
+        pSrcSurface->Format,
+        pcRenderParams->pTarget[0]->Format,
+        pcRenderParams->pCompAlpha,
+        pSrcSurface->pDeinterlaceParams,
+        pSrcSurface->bQueryVariance);
+
     //!
     //! \brief SFC can be the output pipe when the following conditions are all met
     //!        1.  User feature keys value "SFC Disable" is false
@@ -252,6 +287,12 @@ bool VphalSfcState::IsOutputPipeSfcFeasible(
          (pSrcSurface->Format != Format_A8R8G8B8 && pSrcSurface->Format != Format_A8B8G8R8))    &&
         pSrcSurface->bQueryVariance             == false)
     {
+        // For platforms with VEBOX disabled but procamp enabled, go Render path
+        if (MEDIA_IS_SKU(m_renderHal->pSkuTable, FtrDisableVEBoxFeatures) && pSrcSurface->pProcampParams != nullptr)
+        {
+            return false;
+        }
+
         return true;
     }
 
@@ -636,6 +677,16 @@ void VphalSfcState::SetRenderingFlags(
     // Cache Render Target pointer
     pRenderData->pRenderTarget = pRenderTarget;
 
+    VPHAL_RENDER_NORMALMESSAGE(
+        "RenderData: bScaling %d, bColorFill %d, bIEF %d, SfcInputFormat %d, SfcRotation %d, SfcScalingMode %d, SfcSrcChromaSiting %d",
+        m_renderData.bScaling,
+        m_renderData.bColorFill,
+        m_renderData.bIEF,
+        m_renderData.SfcInputFormat,
+        m_renderData.SfcRotation,
+        m_renderData.SfcScalingMode,
+        m_renderData.SfcSrcChromaSiting);
+
 finish:
     return;
 }
@@ -682,8 +733,7 @@ bool VphalSfcState::IsFormatSupported(
     // Check if the input/output combination is supported, given certain alpha fill mode.
     // So far SFC only supports filling constant alpha.
     if (pAlphaParams &&
-        (pAlphaParams->AlphaMode == VPHAL_ALPHA_FILL_MODE_NONE ||
-         pAlphaParams->AlphaMode == VPHAL_ALPHA_FILL_MODE_SOURCE_STREAM))
+        pAlphaParams->AlphaMode == VPHAL_ALPHA_FILL_MODE_SOURCE_STREAM)
     {
         if ((pOutSurface->Format == Format_A8R8G8B8    ||
              pOutSurface->Format == Format_A8B8G8R8    ||
@@ -923,6 +973,11 @@ MOS_STATUS VphalSfcState::SetSfcStateParams(
             break;
     }
 
+    VPHAL_RENDER_NORMALMESSAGE("SfcStateParams: dwInputChromaSubSampling %d, b8tapChromafiltering %d, dwChromaDownSamplingMode %d.",
+        pSfcStateParams->dwInputChromaSubSampling,
+        pSfcStateParams->b8tapChromafiltering,
+        pSfcStateParams->dwChromaDownSamplingMode);
+
     SetSfcStateInputOrderingMode(pRenderData, pSfcStateParams);
 
     pSfcStateParams->OutputFrameFormat = pOutSurface->Format;
@@ -1055,6 +1110,11 @@ MOS_STATUS VphalSfcState::SetSfcStateParams(
         pSfcStateParams->bMirrorEnable = true;
     }
 
+    VPHAL_RENDER_NORMALMESSAGE("SfcStateParams: dwMirrorType %d, RotationMode %d, bMirrorEnable %d.",
+        pSfcStateParams->dwMirrorType,
+        pSfcStateParams->RotationMode,
+        pSfcStateParams->bMirrorEnable);
+
     // ColorFill params
     if (m_renderData.bColorFill)
     {
@@ -1155,6 +1215,11 @@ MOS_STATUS VphalSfcState::SetSfcStateParams(
         pSfcStateParams->bInputColorSpace = false;
     }
 
+    VPHAL_RENDER_NORMALMESSAGE("SfcStateParams: bCSCEnable %d, bRGBASwapEnable %d, bMirrorEnable %d.",
+        pSfcStateParams->bCSCEnable,
+        pSfcStateParams->bRGBASwapEnable,
+        pSfcStateParams->bMirrorEnable);
+
     // Set MMC status
     VPHAL_RENDER_CHK_STATUS(SetSfcMmcStatus(
         pRenderData,
@@ -1219,6 +1284,10 @@ MOS_STATUS VphalSfcState::SetSfcMmcStatus(
             sfcStateParams->MMCMode    = MOS_MMC_DISABLED;
         }
 
+        VPHAL_RENDER_NORMALMESSAGE("SfcStateParams: bMMCEnable %d, MMCMode %d.",
+            sfcStateParams->bMMCEnable,
+            sfcStateParams->MMCMode);
+
         // Set mmc status output surface for output surface
         m_osInterface->pfnSetMemoryCompressionMode(m_osInterface, &outSurface->OsResource, MOS_MEMCOMP_STATE(sfcStateParams->MMCMode));
     }
