@@ -34,6 +34,7 @@
 #include "codechal_decode_vc1_g11.h"
 #include "codechal_secure_decode_interface.h"
 #include "mhw_vdbox_mfx_g11_X.h"
+#include "hal_oca_interface.h"
 
 MOS_STATUS CodechalDecodeVc1G11::AllocateStandard(
     CodechalSetting *          settings)
@@ -173,6 +174,9 @@ MOS_STATUS CodechalDecodeVc1G11::DecodeStateLevel()
     MOS_COMMAND_BUFFER  cmdBuffer;
     CODECHAL_DECODE_CHK_STATUS_RETURN(m_osInterface->pfnGetCommandBuffer(m_osInterface, &cmdBuffer, 0));
 
+    auto mmioRegisters = m_hwInterface->GetMfxInterface()->GetMmioRegisters(m_vdboxIndex);
+    HalOcaInterface::On1stLevelBBStart(cmdBuffer, *m_osInterface->pOsContext, m_osInterface->CurrentGpuContextHandle, *m_miInterface, *mmioRegisters);
+
     if (m_olpNeeded)
     {
         CODECHAL_DECODE_CHK_STATUS_RETURN(SendPrologWithFrameTracking(&cmdBuffer, false));
@@ -257,12 +261,18 @@ MOS_STATUS CodechalDecodeVc1G11::DecodeStateLevel()
 
     pipeBufAddrParams.pDecodedReconParam = &surfaceParams;
     pipeBufAddrParams.pRawSurfParam = nullptr;
+
+#ifdef _MMC_SUPPORTED
     CODECHAL_DECODE_CHK_STATUS_RETURN(m_mmc->SetPipeBufAddr(&pipeBufAddrParams, &cmdBuffer));
+#endif
+
     pipeBufAddrParams.pDecodedReconParam = nullptr;
 
+#ifdef _MMC_SUPPORTED
     CODECHAL_DECODE_CHK_STATUS_RETURN(m_mmc->CheckReferenceList(&pipeBufAddrParams));
 
     CODECHAL_DECODE_CHK_STATUS_RETURN(m_mmc->SetRefrenceSync(m_disableDecodeSyncLock, m_disableLockForTranscode));
+#endif
 
     CODECHAL_DEBUG_TOOL(
         for (int i = 0; i < CODEC_MAX_NUM_REF_FRAME_NON_AVC; i++)
@@ -713,6 +723,8 @@ submit:
         m_huCCopyInUse = false;
     }
 
+    HalOcaInterface::On1stLevelBBEnd(cmdBuffer, *m_osInterface->pOsContext);
+
     CODECHAL_DECODE_CHK_STATUS_RETURN(m_osInterface->pfnSubmitCommandBuffer(m_osInterface, &cmdBuffer, m_videoContextUsesNullHw));
 
     CODECHAL_DEBUG_TOOL(
@@ -1020,6 +1032,8 @@ MOS_STATUS CodechalDecodeVc1G11::DecodePrimitiveLevelIT()
         m_huCCopyInUse = false;
     }
 
+    HalOcaInterface::On1stLevelBBEnd(cmdBuffer, *m_osInterface->pOsContext);
+
     CODECHAL_DECODE_CHK_STATUS_RETURN(m_osInterface->pfnSubmitCommandBuffer(m_osInterface, &cmdBuffer, m_videoContextUsesNullHw));
 
     CODECHAL_DEBUG_TOOL(
@@ -1088,7 +1102,7 @@ MOS_STATUS CodechalDecodeVc1G11::HandleSkipFrame()
     MHW_GENERIC_PROLOG_PARAMS           genericPrologParams;
     MOS_SURFACE                         srcSurface;
     uint8_t                             fwdRefIdx;
-    uint16_t                            surfaceHeight;
+    uint32_t                            surfaceSize;
     MOS_SYNC_PARAMS                     syncParams;
     MOS_STATUS                          eStatus = MOS_STATUS_SUCCESS;
 
@@ -1101,19 +1115,24 @@ MOS_STATUS CodechalDecodeVc1G11::HandleSkipFrame()
     srcSurface.OsResource = m_vc1RefList[fwdRefIdx]->resRefPic;
     CODECHAL_DECODE_CHK_STATUS_RETURN(CodecHalGetResourceInfo(m_osInterface, &srcSurface));
 
+#ifdef _MMC_SUPPORTED
     CODECHAL_DECODE_CHK_STATUS_RETURN(m_mmc->SetSurfaceMmcMode(&m_destSurface, &srcSurface));
+#endif
 
-    surfaceHeight = MOS_ALIGN_CEIL(((srcSurface.dwHeight * 3) / 2), MOS_YTILE_H_ALIGNMENT);
+        surfaceSize = ((srcSurface.OsResource.pGmmResInfo->GetArraySize()) > 1) ?
+            ((uint32_t)(srcSurface.OsResource.pGmmResInfo->GetQPitchPlanar(GMM_PLANE_Y) *
+                        srcSurface.OsResource.pGmmResInfo->GetRenderPitch())) :
+            (uint32_t)(srcSurface.OsResource.pGmmResInfo->GetSizeMainSurface());
 
     if (m_hwInterface->m_noHuC)
     {
         CodechalDataCopyParams dataCopyParams;
         MOS_ZeroMemory(&dataCopyParams, sizeof(CodechalDataCopyParams));
         dataCopyParams.srcResource = &srcSurface.OsResource;
-        dataCopyParams.srcSize     = surfaceHeight * m_destSurface.dwPitch;
+        dataCopyParams.srcSize     = surfaceSize;
         dataCopyParams.srcOffset = srcSurface.dwOffset;
         dataCopyParams.dstResource = &m_destSurface.OsResource;
-        dataCopyParams.dstSize     = surfaceHeight * m_destSurface.dwPitch;
+        dataCopyParams.dstSize     = surfaceSize;
         dataCopyParams.dstOffset   = m_destSurface.dwOffset;
 
         CODECHAL_DECODE_CHK_STATUS_RETURN(m_hwInterface->CopyDataSourceWithDrv(&dataCopyParams));
@@ -1146,7 +1165,7 @@ MOS_STATUS CodechalDecodeVc1G11::HandleSkipFrame()
             &cmdBuffer,                             // pCmdBuffer
             &srcSurface.OsResource,                 // presSrc
             &m_destSurface.OsResource,              // presDst
-            surfaceHeight * m_destSurface.dwPitch,  // u32CopyLength
+            surfaceSize,                            // u32CopyLength
             srcSurface.dwOffset,                    // u32CopyInputOffset
             m_destSurface.dwOffset));               // u32CopyOutputOffset
 
@@ -1231,7 +1250,9 @@ MOS_STATUS CodechalDecodeVc1G11::PerformVc1Olp()
 
     CodecHalGetResourceInfo(m_osInterface, &m_deblockSurface);  // DstSurface
 
+#ifdef _MMC_SUPPORTED
     CODECHAL_DECODE_CHK_STATUS_RETURN(m_mmc->DisableSurfaceMmcState(&m_deblockSurface));
+#endif
 
     CODECHAL_DECODE_CHK_STATUS_RETURN(stateHeapInterface->pfnRequestSshSpaceForCmdBuf(
         stateHeapInterface,
@@ -1299,7 +1320,9 @@ MOS_STATUS CodechalDecodeVc1G11::PerformVc1Olp()
     surfaceParamsSrc.dwYOffset[MHW_U_PLANE] =
         (m_destSurface.UPlaneOffset.iYOffset % MOS_YTILE_H_ALIGNMENT);
 
+#ifdef _MMC_SUPPORTED
     CODECHAL_DECODE_CHK_STATUS_RETURN(m_mmc->GetSurfaceMmcState(surfaceParamsSrc.psSurface));
+#endif
 
     MHW_RCS_SURFACE_PARAMS surfaceParamsDst;
     MOS_ZeroMemory(&surfaceParamsDst, sizeof(surfaceParamsDst));
