@@ -1881,7 +1881,8 @@ MOS_STATUS CodechalEncodeMpeg2G11::SendMbEncSurfaces(
 
 MOS_STATUS CodechalEncodeMpeg2G11::SendPrologWithFrameTracking(
     PMOS_COMMAND_BUFFER         cmdBuffer,
-    bool                        frameTracking)
+    bool                        frameTracking,
+    MHW_MI_MMIOREGISTERS       *mmioRegister)
 {
     if (MOS_VE_SUPPORTED(m_osInterface))
     {
@@ -1894,7 +1895,7 @@ MOS_STATUS CodechalEncodeMpeg2G11::SendPrologWithFrameTracking(
         }
     }
 
-    return CodechalEncoderState::SendPrologWithFrameTracking(cmdBuffer, frameTracking);
+    return CodechalEncoderState::SendPrologWithFrameTracking(cmdBuffer, frameTracking, mmioRegister);
 }
 
 MOS_STATUS CodechalEncodeMpeg2G11::ExecuteKernelFunctions()
@@ -2080,7 +2081,7 @@ MOS_STATUS CodechalEncodeMpeg2G11::ExecuteKernelFunctions()
                 CODECHAL_MEDIA_STATE_ENC_QUALITY));
         }
         CODECHAL_ENCODE_CHK_STATUS_RETURN(m_debugInterface->DumpBuffer(
-            &m_brcBuffers.resBrcImageStatesReadBuffer[m_currRecycledBufIdx],
+            &m_brcBuffers.resBrcImageStatesWriteBuffer,
             CodechalDbgAttr::attrOutput,
             "ImgStateWrite",
             BRC_IMG_STATE_SIZE_PER_PASS * m_hwInterface->GetMfxInterface()->GetBrcNumPakPasses(),
@@ -2094,16 +2095,6 @@ MOS_STATUS CodechalEncodeMpeg2G11::ExecuteKernelFunctions()
             m_brcHistoryBufferSize,
             0,
             CODECHAL_MEDIA_STATE_BRC_UPDATE));
-        if (!Mos_ResourceIsNull(&m_brcBuffers.sBrcMbQpBuffer.OsResource))
-        {
-            CODECHAL_ENCODE_CHK_STATUS_RETURN(m_debugInterface->DumpBuffer(
-                &m_brcBuffers.resBrcPakStatisticBuffer[m_brcPakStatisticsSize],
-                CodechalDbgAttr::attrOutput,
-                "MbQp",
-                m_brcBuffers.sBrcMbQpBuffer.dwPitch*m_brcBuffers.sBrcMbQpBuffer.dwHeight,
-                m_brcBuffers.dwBrcMbQpBottomFieldOffset,
-                CODECHAL_MEDIA_STATE_BRC_UPDATE));
-        }
         if (m_brcBuffers.pMbEncKernelStateInUse)
         {
             CODECHAL_ENCODE_CHK_STATUS_RETURN(m_debugInterface->DumpCurbe(
