@@ -25,6 +25,7 @@
 //!
 
 #include "media_libva_decoder.h"
+#include "media_libva_vp.h"
 #include "media_libva_util.h"
 #include "media_ddi_decode_base.h"
 #include "codechal.h"
@@ -39,6 +40,15 @@ DdiMediaDecode::DdiMediaDecode(DDI_DECODE_CONFIG_ATTR *ddiDecodeAttr)
     m_ddiDecodeAttr = ddiDecodeAttr;
     m_ddiDecodeCtx  = nullptr;
     MOS_ZeroMemory(&m_destSurface, sizeof(m_destSurface));
+    m_groupIndex = 0;
+    m_picWidthInMB = 0;
+    m_picHeightInMB = 0;
+    m_decProcessingType = 0;
+    m_width = 0;
+    m_height = 0;
+    m_streamOutEnabled = false;
+    m_sliceParamBufNum = 0;
+    m_sliceCtrlBufNum = 0;
     m_codechalSettings = CodechalSetting::CreateCodechalSetting();
 }
 
@@ -90,6 +100,12 @@ VAStatus DdiMediaDecode::ParseProcessingBuffer(
 
     if (m_decProcessingType == VA_DEC_PROCESSING)
     {
+        if(m_procBuf == nullptr)
+        {
+            m_procBuf = (VAProcPipelineParameterBuffer*)MOS_AllocAndZeroMemory(sizeof(VAProcPipelineParameterBuffer));
+            DDI_CHK_NULL(m_procBuf, "nullptr m_procBuf", VA_STATUS_ERROR_ALLOCATION_FAILED);
+            MOS_SecureMemcpy(m_procBuf, sizeof(VAProcPipelineParameterBuffer), procBuf, sizeof(VAProcPipelineParameterBuffer));
+        }
         auto decProcessingParams =
             (PCODECHAL_DECODE_PROCESSING_PARAMS)m_ddiDecodeCtx->DecodeParams.m_procParams;
 
@@ -184,6 +200,14 @@ VAStatus DdiMediaDecode::BeginPicture(
     /* As it is already checked in the upper caller, skip the check */
     mediaCtx = DdiMedia_GetMediaContext(ctx);
 
+#ifdef _DECODE_PROCESSING_SUPPORTED
+    //renderTarget is decode output surface; set renderTarget as vp sfc input surface m_procBuf->surface = rederTarget
+    if(m_procBuf)
+    {
+        m_procBuf->surface = renderTarget;
+    }
+#endif
+
     DDI_MEDIA_SURFACE *curRT;
     curRT = (DDI_MEDIA_SURFACE *)DdiMedia_GetSurfaceFromVASurfaceID(mediaCtx, renderTarget);
     DDI_CHK_NULL(curRT, "nullptr pCurRT", VA_STATUS_ERROR_INVALID_SURFACE);
@@ -197,6 +221,7 @@ VAStatus DdiMediaDecode::BeginPicture(
     m_ddiDecodeCtx->DecodeParams.m_numSlices       = 0;
     m_ddiDecodeCtx->DecodeParams.m_dataSize        = 0;
     m_ddiDecodeCtx->DecodeParams.m_deblockDataSize = 0;
+    m_ddiDecodeCtx->DecodeParams.m_executeCallIndex = 0;
     m_groupIndex                                   = 0;
 
     // register render targets
@@ -640,6 +665,111 @@ VAStatus DdiMediaDecode::SetDecodeParams()
     return VA_STATUS_SUCCESS;
 }
 
+VAStatus DdiMediaDecode::ExtraDownScaling(
+        VADriverContextP         ctx,
+        VAContextID              context)
+{
+#ifdef _DECODE_PROCESSING_SUPPORTED
+    DDI_CHK_NULL(ctx, "nullptr ctx", VA_STATUS_ERROR_INVALID_CONTEXT);
+    VAStatus vaStatus = MOS_STATUS_SUCCESS;
+    PDDI_MEDIA_CONTEXT mediaCtx = DdiMedia_GetMediaContext(ctx);
+    DDI_CHK_NULL(mediaCtx, "nullptr ctx", VA_STATUS_ERROR_INVALID_CONTEXT);
+    DDI_CHK_NULL(m_ddiDecodeCtx, "nullptr ctx", VA_STATUS_ERROR_INVALID_CONTEXT);
+    CodechalDecode *decoder = dynamic_cast<CodechalDecode *>(m_ddiDecodeCtx->pCodecHal);
+    DDI_CHK_NULL(decoder, "nullptr decoder", VA_STATUS_ERROR_INVALID_PARAMETER);
+    if(m_ddiDecodeCtx->DecodeParams.m_procParams &&
+        !decoder->IsVdSfcSupported())
+    {
+        //check vp context
+        VAContextID vpCtxID = VA_INVALID_ID;
+        if (mediaCtx->pVpCtxHeap != nullptr && mediaCtx->pVpCtxHeap->pHeapBase != nullptr)
+        {
+            //Get VP Context from heap.
+            vpCtxID = (VAContextID)(0 + DDI_MEDIA_VACONTEXTID_OFFSET_VP);
+        }
+        else
+        {
+            //Create VP Context.
+            vaStatus = DdiVp_CreateContext(ctx, 0, 0, 0, 0, 0, 0, &vpCtxID);
+            DDI_CHK_RET(vaStatus, "Create VP Context failed.");
+        }
+
+        uint32_t        ctxType;
+        PDDI_VP_CONTEXT pVpCtx = (PDDI_VP_CONTEXT)DdiMedia_GetContextFromContextID(ctx, vpCtxID, &ctxType);
+        DDI_CHK_NULL(pVpCtx, "nullptr pVpCtx", VA_STATUS_ERROR_INVALID_CONTEXT);
+
+        //Set parameters
+        VAProcPipelineParameterBuffer* pInputPipelineParam = m_procBuf;
+        DDI_CHK_NULL(pInputPipelineParam, "nullptr pInputPipelineParam", VA_STATUS_ERROR_ALLOCATION_FAILED);
+
+        vaStatus = DdiVp_BeginPicture(ctx, vpCtxID, pInputPipelineParam->additional_outputs[0]);
+        DDI_CHK_RET(vaStatus, "VP BeginPicture failed");
+
+        vaStatus = DdiVp_SetProcPipelineParams(ctx, pVpCtx, pInputPipelineParam);
+        DDI_CHK_RET(vaStatus, "VP SetProcPipelineParams failed.");
+
+        vaStatus = DdiVp_EndPicture(ctx, vpCtxID);
+        DDI_CHK_RET(vaStatus, "VP EndPicture failed.");
+    }
+#endif
+    return MOS_STATUS_SUCCESS;
+}
+
+VAStatus DdiMediaDecode::InitDummyReference(CodechalDecode& decoder)
+{
+    PMOS_SURFACE dummyReference = decoder.GetDummyReference();
+
+    // If dummy reference is from decode output surface, need to update frame by frame
+    if (decoder.GetDummyReferenceStatus() == CODECHAL_DUMMY_REFERENCE_DEST_SURFACE)
+    {
+        MOS_ZeroMemory(dummyReference, sizeof(MOS_SURFACE));
+        decoder.SetDummyReferenceStatus(CODECHAL_DUMMY_REFERENCE_INVALID);
+    }
+
+    if (!Mos_ResourceIsNull(&dummyReference->OsResource))
+    {
+        Mos_Specific_GetResourceInfo(decoder.GetOsInterface(), &dummyReference->OsResource, dummyReference);
+
+        // Check if need to re-get dummy reference from DPB or re-allocated
+        if (dummyReference->dwWidth < m_ddiDecodeCtx->DecodeParams.m_destSurface->dwWidth ||
+            dummyReference->dwHeight < m_ddiDecodeCtx->DecodeParams.m_destSurface->dwHeight)
+        {
+            // Check if the dummy reference needs to be re-allocated
+            if (decoder.GetDummyReferenceStatus() == CODECHAL_DUMMY_REFERENCE_ALLOCATED)
+            {
+                decoder.GetOsInterface()->pfnFreeResource(decoder.GetOsInterface(), &dummyReference->OsResource);
+            }
+
+            // Reset dummy reference
+            MOS_ZeroMemory(dummyReference, sizeof(MOS_SURFACE));
+            decoder.SetDummyReferenceStatus(CODECHAL_DUMMY_REFERENCE_INVALID);
+
+            // Considering potential risk, disable the dummy reference from DPB path temporarily
+            //GetDummyReferenceFromDPB(m_ddiDecodeCtx);
+
+            //if (!Mos_ResourceIsNull(&dummyReference->OsResource))
+            //{
+            //    decoder->SetDummyReferenceStatus(CODECHAL_DUMMY_REFERENCE_DPB);
+            //}
+        }
+    }
+    else
+    {
+        // Init dummy reference
+        MOS_ZeroMemory(dummyReference, sizeof(MOS_SURFACE));
+        decoder.SetDummyReferenceStatus(CODECHAL_DUMMY_REFERENCE_INVALID);
+
+        // Considering potential risk, disable the dummy reference from DPB path temporarily
+        //GetDummyReferenceFromDPB(m_ddiDecodeCtx);
+        //if (!Mos_ResourceIsNull(&dummyReference->OsResource))
+        //{
+        //    decoder->SetDummyReferenceStatus(CODECHAL_DUMMY_REFERENCE_DPB);
+        //}
+    }
+
+    return VA_STATUS_SUCCESS;
+}
+
 VAStatus DdiMediaDecode::EndPicture(
     VADriverContextP ctx,
     VAContextID      context)
@@ -663,61 +793,14 @@ VAStatus DdiMediaDecode::EndPicture(
 
     if (MEDIA_IS_WA(&m_ddiDecodeCtx->pMediaCtx->WaTable, WaDummyReference))
     {
-        CodechalDecode *decoder = dynamic_cast<CodechalDecode *>(m_ddiDecodeCtx->pCodecHal);
-        PMOS_SURFACE dummyReference = decoder->GetDummyReference();
-
-        // If dummy reference is from decode output surface, need to update frame by frame
-        if (decoder->GetDummyReferenceStatus() == CODECHAL_DUMMY_REFERENCE_DEST_SURFACE)
-        {
-            MOS_ZeroMemory(dummyReference, sizeof(MOS_SURFACE));
-            decoder->SetDummyReferenceStatus(CODECHAL_DUMMY_REFERENCE_INVALID);
-        }
-
         Mos_Specific_GetResourceInfo(
             m_ddiDecodeCtx->pCodecHal->GetOsInterface(), 
             &m_ddiDecodeCtx->DecodeParams.m_destSurface->OsResource,
             m_ddiDecodeCtx->DecodeParams.m_destSurface);
 
-        if (!Mos_ResourceIsNull(&dummyReference->OsResource))
-        {
-            Mos_Specific_GetResourceInfo(decoder->GetOsInterface(), &dummyReference->OsResource, dummyReference);
-
-            // Check if need to re-get dummy reference from DPB or re-allocated
-            if (dummyReference->dwWidth < m_ddiDecodeCtx->DecodeParams.m_destSurface->dwWidth ||
-                dummyReference->dwHeight < m_ddiDecodeCtx->DecodeParams.m_destSurface->dwHeight)
-            {
-                // Check if the dummy reference needs to be re-allocated
-                if (decoder->GetDummyReferenceStatus() == CODECHAL_DUMMY_REFERENCE_ALLOCATED)
-                {
-                    decoder->GetOsInterface()->pfnFreeResource(decoder->GetOsInterface(), &dummyReference->OsResource);
-                }
-
-                // Reset dummy reference
-                MOS_ZeroMemory(dummyReference, sizeof(MOS_SURFACE));
-                decoder->SetDummyReferenceStatus(CODECHAL_DUMMY_REFERENCE_INVALID);
-
-                // Considering potential risk, disable the dummy reference from DPB path temporarily
-                //GetDummyReferenceFromDPB(m_ddiDecodeCtx);
-
-                //if (!Mos_ResourceIsNull(&dummyReference->OsResource))
-                //{
-                //    decoder->SetDummyReferenceStatus(CODECHAL_DUMMY_REFERENCE_DPB);
-                //}
-            }
-        }
-        else
-        {
-            // Init dummy reference
-            MOS_ZeroMemory(dummyReference, sizeof(MOS_SURFACE));
-            decoder->SetDummyReferenceStatus(CODECHAL_DUMMY_REFERENCE_INVALID);
-                
-            // Considering potential risk, disable the dummy reference from DPB path temporarily
-            //GetDummyReferenceFromDPB(m_ddiDecodeCtx);
-            //if (!Mos_ResourceIsNull(&dummyReference->OsResource))
-            //{
-            //    decoder->SetDummyReferenceStatus(CODECHAL_DUMMY_REFERENCE_DPB);
-            //}
-        }
+        CodechalDecode *decoder = dynamic_cast<CodechalDecode *>(m_ddiDecodeCtx->pCodecHal);
+        DDI_CHK_NULL(decoder, "Null decoder", VA_STATUS_ERROR_INVALID_PARAMETER);
+        DDI_CHK_RET(InitDummyReference(*decoder), "InitDummyReference failed!");
     }
 
     MOS_STATUS status = m_ddiDecodeCtx->pCodecHal->Execute((void *)(&m_ddiDecodeCtx->DecodeParams));
@@ -727,6 +810,8 @@ VAStatus DdiMediaDecode::EndPicture(
         return VA_STATUS_ERROR_DECODING_ERROR;
     }
 
+    m_ddiDecodeCtx->DecodeParams.m_executeCallIndex++;
+
     (&(m_ddiDecodeCtx->RTtbl))->pCurrentRT = nullptr;
 
     status = m_ddiDecodeCtx->pCodecHal->EndFrame();
@@ -734,6 +819,14 @@ VAStatus DdiMediaDecode::EndPicture(
     {
         return VA_STATUS_ERROR_DECODING_ERROR;
     }
+
+#ifdef _DECODE_PROCESSING_SUPPORTED
+    if (ExtraDownScaling(ctx,context) != VA_STATUS_SUCCESS)
+    {
+        return VA_STATUS_ERROR_DECODING_ERROR;
+    }
+#endif
+
     DDI_FUNCTION_EXIT(VA_STATUS_SUCCESS);
     return VA_STATUS_SUCCESS;
 }
@@ -959,6 +1052,8 @@ VAStatus DdiMediaDecode::CreateCodecHal(
     }
     m_ddiDecodeCtx->pCodecHal = codecHal;
 
+    m_codechalSettings->enableCodecMmc = false;
+    m_codechalSettings->sfcInUseHinted = true;
     if (codecHal->Allocate(m_codechalSettings) != MOS_STATUS_SUCCESS)
     {
         DDI_ASSERTMESSAGE("Failure in decode allocate.\n");
@@ -1038,6 +1133,11 @@ void DdiMediaDecode::GetDummyReferenceFromDPB(
     if (i < DDI_MEDIA_MAX_SURFACE_NUMBER_CONTEXT)
     {
         CodechalDecode *decoder = dynamic_cast<CodechalDecode *>(decodeCtx->pCodecHal);
+        if (decoder == nullptr)
+        {
+            DDI_ASSERTMESSAGE("Codechal decode context is NULL.\n");
+            return;
+        }
         decoder->GetDummyReference()->OsResource = dummyReference.OsResource;
     }
 }
