@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2018, Intel Corporation
+* Copyright (c) 2018-2019, Intel Corporation
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
@@ -30,6 +30,8 @@
 #include "mos_commandbuffer_specific.h"
 #include "mos_util_devult_specific.h"
 #include "mos_cmdbufmgr.h"
+#include "mos_os_virtualengine.h"
+#include <unistd.h>
 
 #define MI_BATCHBUFFER_END 0x05000000
 static pthread_mutex_t command_dump_mutex = PTHREAD_MUTEX_INITIALIZER;
@@ -46,6 +48,7 @@ GpuContextSpecific::GpuContextSpecific(
     m_cmdBufMgr            = cmdBufMgr;
     m_gpuContext           = mosGpuCtx;
     m_statusBufferResource = nullptr;
+    m_maxPatchLocationsize = PATCHLOCATIONLIST_SIZE;
 
     if (reusedContext)
     {
@@ -125,17 +128,22 @@ MOS_STATUS GpuContextSpecific::Init(OsContext *osContext,
         m_createOptionEnhanced->UsingSFC = createOptionEnhanced->UsingSFC;
     }
 
+    for (int i=0; i<MAX_ENGINE_INSTANCE_NUM+1; i++)
+    {
+        m_i915Context[i] = nullptr;
+    }
+
     if (osInterface->ctxBasedScheduling)
     {
-        m_i915Context = mos_gem_context_create_shared(osInterface->pOsContext->bufmgr,
+        m_i915Context[0] = mos_gem_context_create_shared(osInterface->pOsContext->bufmgr,
                                              osInterface->pOsContext->intel_context,
                                              I915_CONTEXT_CREATE_FLAGS_SINGLE_TIMELINE);
-        if (m_i915Context == nullptr)
+        if (m_i915Context[0] == nullptr)
         {
             MOS_OS_ASSERTMESSAGE("Failed to create context.\n");
             return MOS_STATUS_UNKNOWN;
         }
-        m_i915Context->pOsContext = osInterface->pOsContext;
+        m_i915Context[0]->pOsContext = osInterface->pOsContext;
 
         m_i915ExecFlag = I915_EXEC_DEFAULT;
         if (GpuNode == MOS_GPU_NODE_3D || GpuNode == MOS_GPU_NODE_COMPUTE)
@@ -144,7 +152,7 @@ MOS_STATUS GpuContextSpecific::Init(OsContext *osContext,
             engine_map.engine_class = I915_ENGINE_CLASS_RENDER;
             engine_map.engine_instance = 0;
 
-            if (mos_set_context_param_load_balance(m_i915Context,&engine_map, 1))
+            if (mos_set_context_param_load_balance(m_i915Context[0],&engine_map, 1))
             {
                 MOS_OS_ASSERTMESSAGE("Failed to set balancer extension.\n");
                 return MOS_STATUS_UNKNOWN;
@@ -157,7 +165,7 @@ MOS_STATUS GpuContextSpecific::Init(OsContext *osContext,
                 sseu.flags = I915_CONTEXT_SSEU_FLAG_ENGINE_INDEX;
                 sseu.engine.engine_instance = m_i915ExecFlag;
 
-                if (mos_get_context_param_sseu(m_i915Context, &sseu))
+                if (mos_get_context_param_sseu(m_i915Context[0], &sseu))
                 {
                     MOS_OS_ASSERTMESSAGE("Failed to get sseu configuration.");
                     return MOS_STATUS_UNKNOWN;
@@ -169,7 +177,7 @@ MOS_STATUS GpuContextSpecific::Init(OsContext *osContext,
                             mos_hweight8(sseu.subslice_mask)-createOption->packed.SubSliceCount);
                 }
 
-                if (mos_set_context_param_sseu(m_i915Context, sseu))
+                if (mos_set_context_param_sseu(m_i915Context[0], sseu))
                 {
                     MOS_OS_ASSERTMESSAGE("Failed to set sseu configuration.");
                     return MOS_STATUS_UNKNOWN;
@@ -196,11 +204,51 @@ MOS_STATUS GpuContextSpecific::Init(OsContext *osContext,
                 return MOS_STATUS_UNKNOWN;
             }
 
-            if (mos_set_context_param_load_balance(m_i915Context, engine_map, nengine))
+            if (mos_set_context_param_load_balance(m_i915Context[0], engine_map, nengine))
             {
                 MOS_OS_ASSERTMESSAGE("Failed to set balancer extension.\n");
                 return MOS_STATUS_UNKNOWN;
             }
+
+            if (nengine >= 2)
+            {
+                //master queue
+                m_i915Context[1] = mos_gem_context_create_shared(osInterface->pOsContext->bufmgr,
+                                                                 osInterface->pOsContext->intel_context,
+                                                                 I915_CONTEXT_CREATE_FLAGS_SINGLE_TIMELINE);
+                if (m_i915Context[1] == nullptr)
+                {
+                    MOS_OS_ASSERTMESSAGE("Failed to create master context.\n");
+                    return MOS_STATUS_UNKNOWN;
+                }
+                m_i915Context[1]->pOsContext = osInterface->pOsContext;
+
+                if (mos_set_context_param_load_balance(m_i915Context[1], engine_map, 1))
+                {
+                    MOS_OS_ASSERTMESSAGE("Failed to set master context bond extension.\n");
+                    return MOS_STATUS_UNKNOWN;
+                }
+
+                //slave queue
+                for (int i=1; i<nengine; i++)
+                {
+                    m_i915Context[i+1] = mos_gem_context_create_shared(osInterface->pOsContext->bufmgr,
+                                                                     osInterface->pOsContext->intel_context,
+                                                                     I915_CONTEXT_CREATE_FLAGS_SINGLE_TIMELINE);
+                    if (m_i915Context[i+1] == nullptr)
+                    {
+                        MOS_OS_ASSERTMESSAGE("Failed to create slave context.\n");
+                        return MOS_STATUS_UNKNOWN;
+                    }
+                    m_i915Context[i+1]->pOsContext = osInterface->pOsContext;
+
+                    if (mos_set_context_param_bond(m_i915Context[i+1], engine_map[0],&engine_map[i], 1))
+                    {
+                        MOS_OS_ASSERTMESSAGE("Failed to set slave context bond extension.\n");
+                        return MOS_STATUS_UNKNOWN;
+                    }
+                }
+            }
         }
         else
         {
@@ -251,10 +299,14 @@ void GpuContextSpecific::Clear()
     MOS_SafeFreeMemory(m_attachedResources);
     MOS_SafeFreeMemory(m_writeModeList);
     MOS_SafeFreeMemory(m_createOptionEnhanced);
-    if (m_i915Context)
+
+    for (int i=0; i<MAX_ENGINE_INSTANCE_NUM; i++)
     {
-        mos_gem_context_destroy(m_i915Context);
-        m_i915Context = nullptr;
+        if (m_i915Context[i])
+        {
+            mos_gem_context_destroy(m_i915Context[i]);
+            m_i915Context[i] = nullptr;
+        }
     }
 }
 
@@ -325,6 +377,8 @@ MOS_STATUS GpuContextSpecific::SetPatchEntry(
     m_patchLocationList[m_currentNumPatchLocations].AllocationOffset = params->uiResourceOffset;
     m_patchLocationList[m_currentNumPatchLocations].PatchOffset      = params->uiPatchOffset;
     m_patchLocationList[m_currentNumPatchLocations].uiWriteOperation = params->bWrite ? true: false;
+    m_patchLocationList[m_currentNumPatchLocations].cmdBo            =
+                params->cmdBuffer != nullptr ? params->cmdBuffer->OsResource.bo : nullptr;
 
     if (osInterface->osCpInterface &&
         osInterface->osCpInterface->IsHMEnabled())
@@ -359,7 +413,15 @@ MOS_STATUS GpuContextSpecific::GetCommandBuffer(
     MOS_STATUS      eStatus = MOS_STATUS_SUCCESS;
     CommandBuffer* cmdBuf = nullptr;
 
-    if (m_cmdBufFlushed)
+    uint32_t secondaryIdx = flags;
+    bool isPrimaryCmdBuffer = (secondaryIdx == 0);
+    bool hasSecondaryCmdBuffer = (!isPrimaryCmdBuffer &&
+                               (m_secondaryCmdBufs.count(secondaryIdx) != 0));
+
+    bool needToAlloc = ((isPrimaryCmdBuffer && m_cmdBufFlushed) ||
+                        (!isPrimaryCmdBuffer && !hasSecondaryCmdBuffer));
+
+    if (needToAlloc)
     {
         MOS_LockMutex(m_cmdBufPoolMutex);
         if (m_cmdBufPool.size() < MAX_CMD_BUF_NUM)
@@ -425,17 +487,29 @@ MOS_STATUS GpuContextSpecific::GetCommandBuffer(
         comamndBuffer->iRemaining = cmdBuf->GetCmdBufSize();
         comamndBuffer->iCmdIndex  = m_nextFetchIndex;
         comamndBuffer->iVdboxNodeIndex = MOS_VDBOX_NODE_INVALID;
+        comamndBuffer->iVeboxNodeIndex = MOS_VEBOX_NODE_INVALID;
         comamndBuffer->Attributes.pAttriVe = nullptr;
 
         // zero comamnd buffer
         MOS_ZeroMemory(comamndBuffer->pCmdBase, comamndBuffer->iRemaining);
+        comamndBuffer->iSubmissionType = SUBMISSION_TYPE_SINGLE_PIPE;
         MOS_ZeroMemory(&comamndBuffer->Attributes,sizeof(comamndBuffer->Attributes));
 
-        // update command buffer relared filed in GPU context
-        m_cmdBufFlushed = false;
+        if (isPrimaryCmdBuffer)
+        {
+            // update command buffer relared filed in GPU context
+            m_cmdBufFlushed = false;
 
-        // keep a copy in GPU context
-        MOS_SecureMemcpy(m_commandBuffer, sizeof(MOS_COMMAND_BUFFER), comamndBuffer, sizeof(MOS_COMMAND_BUFFER));
+            // keep a copy in GPU context
+            MOS_SecureMemcpy(m_commandBuffer, sizeof(MOS_COMMAND_BUFFER), comamndBuffer, sizeof(MOS_COMMAND_BUFFER));
+        }
+        else
+        {
+            PMOS_COMMAND_BUFFER tempCmdBuf = (PMOS_COMMAND_BUFFER)MOS_AllocAndZeroMemory(sizeof(MOS_COMMAND_BUFFER));
+            MOS_OS_CHK_NULL_RETURN(tempCmdBuf);
+            m_secondaryCmdBufs[secondaryIdx] = tempCmdBuf;
+            MOS_SecureMemcpy(tempCmdBuf, sizeof(MOS_COMMAND_BUFFER), comamndBuffer, sizeof(MOS_COMMAND_BUFFER));
+        }
 
         // Command buffers are treated as cyclical buffers, the CB after the just submitted one
         // has the minimal fence value that we should wait
@@ -448,10 +522,24 @@ MOS_STATUS GpuContextSpecific::GetCommandBuffer(
     else
     {
         // current command buffer still active, directly copy to comamndBuffer
-        MOS_SecureMemcpy(comamndBuffer, sizeof(MOS_COMMAND_BUFFER), m_commandBuffer, sizeof(MOS_COMMAND_BUFFER));
+        if (isPrimaryCmdBuffer)
+        {
+            MOS_SecureMemcpy(comamndBuffer, sizeof(MOS_COMMAND_BUFFER), m_commandBuffer, sizeof(MOS_COMMAND_BUFFER));
+        }
+        else
+        {
+            MOS_SecureMemcpy(comamndBuffer, sizeof(MOS_COMMAND_BUFFER), m_secondaryCmdBufs[secondaryIdx], sizeof(MOS_COMMAND_BUFFER));
+        }
     }
 
-    MOS_OS_CHK_STATUS_RETURN(RegisterResource(&m_commandBuffer->OsResource, false));
+    if (isPrimaryCmdBuffer)
+    {
+        MOS_OS_CHK_STATUS_RETURN(RegisterResource(&m_commandBuffer->OsResource, false));
+    }
+    else
+    {
+        MOS_OS_CHK_STATUS_RETURN(RegisterResource(&m_secondaryCmdBufs[secondaryIdx]->OsResource, false));
+    }
 
     return MOS_STATUS_SUCCESS;
 }
@@ -465,15 +553,35 @@ void GpuContextSpecific::ReturnCommandBuffer(
     MOS_OS_ASSERT(cmdBuffer);
     MOS_OS_ASSERT(m_commandBuffer);
 
-    m_commandBuffer->iOffset    = cmdBuffer->iOffset;
-    m_commandBuffer->iRemaining = cmdBuffer->iRemaining;
-    m_commandBuffer->pCmdPtr    = cmdBuffer->pCmdPtr;
-    m_commandBuffer->iVdboxNodeIndex = cmdBuffer->iVdboxNodeIndex;
+    bool isPrimaryCmdBuf = (flags == 0);
+
+    if (isPrimaryCmdBuf)
+    {
+        m_commandBuffer->iOffset    = cmdBuffer->iOffset;
+        m_commandBuffer->iRemaining = cmdBuffer->iRemaining;
+        m_commandBuffer->pCmdPtr    = cmdBuffer->pCmdPtr;
+        m_commandBuffer->iVdboxNodeIndex = cmdBuffer->iVdboxNodeIndex;
+        m_commandBuffer->iVeboxNodeIndex = cmdBuffer->iVeboxNodeIndex;
+    }
+    else
+    {
+        uint32_t secondaryIdx = flags;
+        MOS_OS_ASSERT(m_secondaryCmdBufs.count(secondaryIdx));
+
+        MOS_SecureMemcpy(m_secondaryCmdBufs[secondaryIdx], sizeof(MOS_COMMAND_BUFFER), cmdBuffer, sizeof(MOS_COMMAND_BUFFER));
+    }
 }
 
 MOS_STATUS GpuContextSpecific::ResetCommandBuffer()
 {
     m_cmdBufFlushed = true;
+    auto it = m_secondaryCmdBufs.begin();
+    while(it != m_secondaryCmdBufs.end())
+    {
+        MOS_FreeMemory(it->second);
+        it++;
+    }
+    m_secondaryCmdBufs.clear();
     return MOS_STATUS_SUCCESS;
 }
 
@@ -634,6 +742,8 @@ MOS_STATUS GpuContextSpecific::SubmitCommandBuffer(
     uint32_t     execFlag = gpuNode;
     MOS_STATUS   eStatus  = MOS_STATUS_SUCCESS;
     int32_t      ret      = 0;
+    bool         scalaEnabled = false;
+    auto         it           = m_secondaryCmdBufs.begin();
 
     // Command buffer object DRM pointer
     m_cmdBufFlushed = true;
@@ -642,12 +752,51 @@ MOS_STATUS GpuContextSpecific::SubmitCommandBuffer(
     // Map Resource to Aux if needed
     MapResourcesToAuxTable(cmd_bo);
 
+    if (m_secondaryCmdBufs.size() >= 2)
+    {
+        scalaEnabled = true;
+        cmdBuffer->iSubmissionType = SUBMISSION_TYPE_MULTI_PIPE_MASTER;
+    }
+
+    std::vector<PMOS_RESOURCE> mappedResList;
+
     // Now, the patching will be done, based on the patch list.
     for (uint32_t patchIndex = 0; patchIndex < m_currentNumPatchLocations; patchIndex++)
     {
         auto currentPatch = &m_patchLocationList[patchIndex];
         MOS_OS_CHK_NULL_RETURN(currentPatch);
 
+        auto tempCmdBo = currentPatch->cmdBo == nullptr ? cmd_bo : currentPatch->cmdBo;
+
+        // Following are for Nested BB buffer, if it's nested BB, we need to ensure it's locked.
+        if (tempCmdBo != cmd_bo)
+        {
+            bool isSecondaryCmdBuf = false;
+            it = m_secondaryCmdBufs.begin();
+            while(it != m_secondaryCmdBufs.end())
+            {
+                if (it->second->OsResource.bo == tempCmdBo)
+                {
+                    isSecondaryCmdBuf = true;
+                    break;
+                }
+                it++;
+            }
+
+            for(auto allocIdx = 0; allocIdx < m_numAllocations && (!isSecondaryCmdBuf); allocIdx++)
+            {
+                auto tempRes = (PMOS_RESOURCE)m_allocationList[allocIdx].hAllocation;
+                if (tempCmdBo == tempRes->bo)
+                {
+                    GraphicsResource::LockParams param;
+                    param.m_writeRequest = true;
+                    tempRes->pGfxResource->Lock(m_osContext, param);
+                    mappedResList.push_back(tempRes);
+                    break;
+                }
+            }
+        }
+
         // This is the resource for which patching will be done
         auto resource = (PMOS_RESOURCE)m_allocationList[currentPatch->AllocationIndex].hAllocation;
         MOS_OS_CHK_NULL_RETURN(resource);
@@ -658,16 +807,15 @@ MOS_STATUS GpuContextSpecific::SubmitCommandBuffer(
         // We'll simply set alloc_bo to the command buffer's bo pointer.
         MOS_OS_ASSERT(resource->bo);
 
-        auto alloc_bo = (resource->bo) ? resource->bo : cmd_bo;
+        auto alloc_bo = (resource->bo) ? resource->bo : tempCmdBo;
 
         MOS_OS_CHK_STATUS_RETURN(osInterface->osCpInterface->PermeatePatchForHM(
-            cmd_bo->virt,
+            tempCmdBo->virt,
             currentPatch,
             resource));
 
-#ifndef ANDROID
         uint64_t boOffset = alloc_bo->offset64;
-        if (alloc_bo != cmd_bo)
+        if (alloc_bo != tempCmdBo)
         {
             auto item_ctx = osContext->contextOffsetList.begin();
             for (; item_ctx != osContext->contextOffsetList.end(); item_ctx++)
@@ -679,56 +827,68 @@ MOS_STATUS GpuContextSpecific::SubmitCommandBuffer(
                 }
             }
         }
+
         if (osContext->bUse64BitRelocs)
         {
-            *((uint64_t *)((uint8_t *)cmd_bo->virt + currentPatch->PatchOffset)) =
+            *((uint64_t *)((uint8_t *)tempCmdBo->virt + currentPatch->PatchOffset)) =
                     boOffset + currentPatch->AllocationOffset;
         }
         else
         {
-            *((uint32_t *)((uint8_t *)cmd_bo->virt + currentPatch->PatchOffset)) =
+            *((uint32_t *)((uint8_t *)tempCmdBo->virt + currentPatch->PatchOffset)) =
                     boOffset + currentPatch->AllocationOffset;
         }
 
-        // This call will patch the command buffer with the offsets of the indirect state region of the command buffer
-        ret = mos_bo_emit_reloc2(
-            cmd_bo,                                                            // Command buffer
-            currentPatch->PatchOffset,                                         // Offset in the command buffer
-            alloc_bo,                                                          // Allocation object for which the patch will be made.
-            currentPatch->AllocationOffset,                                    // Offset to the indirect state
-            I915_GEM_DOMAIN_RENDER,                                            // Read domain
-            (currentPatch->uiWriteOperation) ? I915_GEM_DOMAIN_RENDER : 0x0,   // Write domain
-            boOffset);
-#else
-            if (osContext->bUse64BitRelocs)
+         if (scalaEnabled)
+        {
+            it = m_secondaryCmdBufs.begin();
+            while(it != m_secondaryCmdBufs.end())
             {
-                *((uint64_t*)((uint8_t*)cmd_bo->virt + currentPatch->PatchOffset)) =
-                        alloc_bo->offset64 + currentPatch->AllocationOffset;
-            }
-            else
-            {
-                *((uint32_t*)((uint8_t*)cmd_bo->virt + currentPatch->PatchOffset)) =
-                        alloc_bo->offset64 + currentPatch->AllocationOffset;
+                if (it->second->OsResource.bo == tempCmdBo &&
+                    it->second->iSubmissionType & SUBMISSION_TYPE_MULTI_PIPE_SLAVE)
+                {
+                    mos_bo_set_exec_object_async(alloc_bo);
+                    break;
+                }
+                it++;
             }
+        }
+        else if (cmdBuffer->iSubmissionType & SUBMISSION_TYPE_MULTI_PIPE_SLAVE)
+        {
+            mos_bo_set_exec_object_async(alloc_bo);
+        }
 
         // This call will patch the command buffer with the offsets of the indirect state region of the command buffer
-        ret = mos_bo_emit_reloc(
-            cmd_bo,                          // Command buffer
-            currentPatch->PatchOffset,       // Offset in the command buffer
-            alloc_bo,                        // Allocation object for which the patch will be made.
-            currentPatch->AllocationOffset,  // Offset to the indirect state
-            I915_GEM_DOMAIN_RENDER,          // Read domain
-            (currentPatch->uiWriteOperation) ? I915_GEM_DOMAIN_RENDER : 0x0);  // Write domain
-#endif
+        if (mos_gem_bo_is_softpin(alloc_bo))
+        {
+            ret = mos_bo_emit_reloc(tempCmdBo, 0, alloc_bo, 0, I915_GEM_DOMAIN_RENDER, I915_GEM_DOMAIN_CPU);
+        }
+        else
+        {
+            ret = mos_bo_emit_reloc2(
+                tempCmdBo,                                                         // Command buffer
+                currentPatch->PatchOffset,                                         // Offset in the command buffer
+                alloc_bo,                                                          // Allocation object for which the patch will be made.
+                currentPatch->AllocationOffset,                                    // Offset to the indirect state
+                I915_GEM_DOMAIN_RENDER,                                            // Read domain
+                (currentPatch->uiWriteOperation) ? I915_GEM_DOMAIN_RENDER : 0x0,   // Write domain
+                boOffset);
+        }
+
         if (ret != 0)
         {
             MOS_OS_ASSERTMESSAGE("Error patching alloc_bo = 0x%x, cmd_bo = 0x%x.",
                 (uintptr_t)alloc_bo,
-                (uintptr_t)cmd_bo);
+                (uintptr_t)tempCmdBo);
             return MOS_STATUS_UNKNOWN;
         }
     }
 
+    for(auto res: mappedResList)
+    {
+        res->pGfxResource->Unlock(m_osContext);
+    }
+
     //Add Batch buffer End Command
     uint32_t batchBufferEndCmd = MI_BATCHBUFFER_END;
     if (MOS_FAILED(Mos_AddCommand(
@@ -744,6 +904,15 @@ MOS_STATUS GpuContextSpecific::SubmitCommandBuffer(
     MOS_OS_CHK_NULL_RETURN(cmdBuffer->OsResource.pGfxResource);
     cmdBuffer->OsResource.pGfxResource->Unlock(m_osContext);
 
+    it = m_secondaryCmdBufs.begin();
+    while(it != m_secondaryCmdBufs.end())
+    {
+        MOS_OS_CHK_NULL_RETURN(it->second->OsResource.pGfxResource);
+        it->second->OsResource.pGfxResource->Unlock(m_osContext);
+
+        it++;
+    }
+
     int32_t perfData;
     if (osContext->pPerfData != nullptr)
     {
@@ -759,7 +928,8 @@ MOS_STATUS GpuContextSpecific::SubmitCommandBuffer(
     int32_t          DR4           = osContext->uEnablePerfTag ? perfData : 0;
 
     //Since CB2 command is not supported, remove it and set cliprects to nullprt as default.
-    if (gpuNode == MOS_GPU_NODE_VIDEO || gpuNode == MOS_GPU_NODE_VIDEO2)
+    if ((gpuNode == MOS_GPU_NODE_VIDEO || gpuNode == MOS_GPU_NODE_VIDEO2) &&
+        (cmdBuffer->iSubmissionType & SUBMISSION_TYPE_SINGLE_PIPE_MASK))
     {
         if (osContext->bKMDHasVCS2)
         {
@@ -871,15 +1041,50 @@ MOS_STATUS GpuContextSpecific::SubmitCommandBuffer(
     }
     else if (nullRendering == false)
     {
-        if (osInterface->ctxBasedScheduling && m_i915Context != nullptr)
+        if (osInterface->ctxBasedScheduling && m_i915Context[0] != nullptr)
         {
-            ret = mos_gem_bo_context_exec2(cmd_bo,
-                m_commandBufferSize,
-                m_i915Context,
-                cliprects,
-                num_cliprects,
-                DR4,
-                m_i915ExecFlag);
+            if (cmdBuffer->iSubmissionType & SUBMISSION_TYPE_MULTI_PIPE_MASK)
+            {
+                if (scalaEnabled)
+                {
+                    uint32_t secondaryIndex = 0;
+                    it = m_secondaryCmdBufs.begin();
+                    while(it != m_secondaryCmdBufs.end())
+                    {
+                        if (it->second->iSubmissionType & SUBMISSION_TYPE_MULTI_PIPE_SLAVE)
+                        {
+                            it->second->iSubmissionType |= (secondaryIndex << SUBMISSION_TYPE_MULTI_PIPE_SLAVE_INDEX_SHIFT);
+                            secondaryIndex++;
+                        }
+
+                        ret = SubmitPipeCommands(it->second,
+                                                 it->second->OsResource.bo,
+                                                 osContext,
+                                                 execFlag,
+                                                 DR4);
+                        it++;
+                    }
+                }
+                else
+                {
+                    ret = SubmitPipeCommands(cmdBuffer,
+                                             cmd_bo,
+                                             osContext,
+                                             execFlag,
+                                             DR4);
+                }
+            }
+            else
+            {
+                ret = mos_gem_bo_context_exec2(cmd_bo,
+                    m_commandBufferSize,
+                    m_i915Context[0],
+                    cliprects,
+                    num_cliprects,
+                    DR4,
+                    m_i915ExecFlag,
+                    nullptr);
+            }
         }
         else
         {
@@ -889,7 +1094,8 @@ MOS_STATUS GpuContextSpecific::SubmitCommandBuffer(
                 cliprects,
                 num_cliprects,
                 DR4,
-                execFlag);
+                execFlag,
+                nullptr);
         }
         if (ret != 0)
         {
@@ -929,6 +1135,14 @@ if (osInterface->bDumpCommandBuffer)
 
     //clear command buffer relocations to fix memory leak issue
     mos_gem_bo_clear_relocs(cmd_bo, 0);
+    it = m_secondaryCmdBufs.begin();
+    while(it != m_secondaryCmdBufs.end())
+    {
+        mos_gem_bo_clear_relocs(it->second->OsResource.bo, 0);
+        MOS_FreeMemory(it->second);
+        it++;
+    }
+    m_secondaryCmdBufs.clear();
 
     // Reset resource allocation
     m_numAllocations = 0;
@@ -942,6 +1156,84 @@ finish:
     return eStatus;
 }
 
+int32_t GpuContextSpecific::SubmitPipeCommands(
+    MOS_COMMAND_BUFFER *cmdBuffer,
+    MOS_LINUX_BO *cmdBo,
+    PMOS_CONTEXT osContext,
+    uint32_t execFlag,
+    int32_t dr4)
+{
+    int32_t      ret        = 0;
+    int          fence      = -1;
+    unsigned int fenceFlag = 0;
+
+    MOS_LINUX_CONTEXT *queue = m_i915Context[0];
+    bool isVeboxSubmission   = false;
+
+    if (execFlag == MOS_GPU_NODE_VIDEO || execFlag == MOS_GPU_NODE_VIDEO2)
+    {
+        execFlag = I915_EXEC_DEFAULT;
+    }
+    if (execFlag == MOS_GPU_NODE_VE)
+    {
+        execFlag = I915_EXEC_DEFAULT;
+        isVeboxSubmission = true;
+    }
+
+    if(cmdBuffer->iSubmissionType & SUBMISSION_TYPE_MULTI_PIPE_SLAVE)
+    {
+        fence = osContext->submit_fence;
+        fenceFlag = I915_EXEC_FENCE_SUBMIT;
+        int slaveIndex = (cmdBuffer->iSubmissionType & SUBMISSION_TYPE_MULTI_PIPE_SLAVE_INDEX_MASK) >> SUBMISSION_TYPE_MULTI_PIPE_SLAVE_INDEX_SHIFT;
+        if(slaveIndex < 7)
+        {
+            queue = m_i915Context[2 + slaveIndex]; //0 is for single pipe, 1 is for master, slave starts from 2
+        }
+        else
+        {
+            MOS_OS_ASSERTMESSAGE("slaveIndex value: %s is invalid!", slaveIndex);
+            return -1;
+        }
+
+        if (isVeboxSubmission)
+        {
+            queue = m_i915Context[cmdBuffer->iVeboxNodeIndex + 1];
+        }
+    }
+
+    //Keep FE and BE0 running on same engine for VT decode
+    if((cmdBuffer->iSubmissionType & SUBMISSION_TYPE_MULTI_PIPE_ALONE)
+        || (cmdBuffer->iSubmissionType & SUBMISSION_TYPE_MULTI_PIPE_MASTER))
+    {
+        if(cmdBuffer->iSubmissionType & SUBMISSION_TYPE_MULTI_PIPE_MASTER)
+        {
+            //Only master pipe needs fence out flag
+            fenceFlag = I915_EXEC_FENCE_OUT;
+        }
+        queue = m_i915Context[1];
+    }
+
+    ret = mos_gem_bo_context_exec2(cmdBo,
+                                  cmdBo->size,
+                                  queue,
+                                  nullptr,
+                                  0,
+                                  dr4,
+                                  execFlag | fenceFlag,
+                                  &fence);
+
+    if(cmdBuffer->iSubmissionType & SUBMISSION_TYPE_MULTI_PIPE_MASTER)
+    {
+        osContext->submit_fence = fence;
+    }
+    if(cmdBuffer->iSubmissionType & SUBMISSION_TYPE_MULTI_PIPE_FLAGS_LAST_PIPE)
+    {
+        close(fence);
+    }
+
+    return ret;
+}
+
 void GpuContextSpecific::IncrementGpuStatusTag()
 {
     m_GPUStatusTag = m_GPUStatusTag % UINT_MAX + 1;
