@@ -1228,6 +1228,7 @@ static MOS_STATUS _UserFeature_SetValue(
 
     if ( (eStatus = _UserFeature_DumpFile(szUserFeatureFile, &pKeyList)) != MOS_STATUS_SUCCESS )
     {
+        MOS_FreeMemory(pKeyList);
         return eStatus;
     }
 
@@ -1330,6 +1331,7 @@ static MOS_STATUS _UserFeature_GetKeyIdbyName(const char  *pcKeyName, void **pUF
     if ( (eStatus = _UserFeature_DumpFile(szUserFeatureFile, &pKeyList)) !=
         MOS_STATUS_SUCCESS )
     {
+        MOS_FreeMemory(pKeyList);
         return eStatus;
     }
 
@@ -1382,6 +1384,7 @@ static MOS_STATUS _UserFeature_GetKeyNamebyId(void  *UFKey, char  *pcKeyName)
         if ( (eStatus = _UserFeature_DumpFile(szUserFeatureFile, &pKeyList)) !=
             MOS_STATUS_SUCCESS )
         {
+            MOS_FreeMemory(pKeyList);
             return eStatus;
         }
 
@@ -2994,6 +2997,15 @@ void MOS_TraceEvent(
     return;
 }
 
+void MOS_TraceDataDump(
+    char * const pcName,
+    uint32_t     flags,
+    void *const  pBuf,
+    uint32_t     dwSize)
+{
+    // not implemented
+}
+
 MOS_STATUS MOS_GfxInfoInit()
 {
     // not implemented
